/*
 * Copyright 2016 Red Hat, Inc. and/or its affiliates
 * and other contributors as indicated by the @author tags.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.keycloak.keys;

import org.keycloak.crypto.KeyWrapper;
import org.keycloak.provider.Provider;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author <a href="mailto:sthorger@redhat.com">Stian Thorgersen</a>
 */
public interface KeyProvider extends Provider {


    /**
     * Returns the {@code KeyWrapper} for a {@code KeyProvider}.
     *
     * @return Returns the {@code KeyWrapper} for a {@code KeyProvider}.
     * @deprecated Use {@link #getKeysStream() getKeysStream} instead.
     */
    @Deprecated
    default List<KeyWrapper> getKeys() {
        return getKeysStream().collect(Collectors.toList());
    }

    /**
     * Returns the {@code KeyWrapper} for a {@code KeyProvider}.
     *
     * @return Returns the {@code KeyWrapper} for a {@code KeyProvider}.
     */
    Stream<KeyWrapper> getKeysStream();

    default void close() {
    }

}
